@@ -286,9 +286,39 @@ def chan_map(dp=None, y_orig='surface', probe_version=None):
                     1st column is channel indices, 2nd x position, 3rd y position
     '''
 
-    dp = Path(dp)
     assert y_orig in ['surface', 'tip']
-    if probe_version is None: probe_version=read_metadata(dp)['probe_version']
+    if probe_version is None:
+        assert dp is not None, "You need to provide either a path or a probe version!"
+        probe_version=read_metadata(dp)['probe_version']
+
+    if probe_version in ['3A', '1.0', '2.0_singleshank']:
+        cm = predefined_chanmap(probe_version)
+    else:
+        probe_version='local'
+
+    if probe_version=='local':
+        if dp is None:
+            raise ValueError("dp argument is not provided - when channel map is \
+                             atypical and probe_version thus called 'local', \
+                             the datapath needs to be provided to load the channel map.")
+        dp = Path(dp)
+        c_ind=np.load(dp/'channel_map.npy');cp=np.load(dp/'channel_positions.npy')
+        cm=npa(np.hstack([c_ind.reshape(max(c_ind.shape),1), cp]), dtype=np.int32)
+
+    if y_orig=='surface':
+        cm[:,1:]=cm[:,1:][::-1]
+
+    return cm
+
+def predefined_chanmap(probe_version='1.0'):
+    '''
+    Returns predefined channel map.
+    Parameters:
+        - probe_version: None, 'local', '3A', '1.0' or '2.0_singleshank' (other types not handled yet, reach out to give your own!).
+    Returns:
+        - chan_map: array of shape (N_electrodes, 3).
+                    1st column is channel indices, 2nd x position, 3rd y position
+    '''
 
     if probe_version in ['3A', '1.0']:
         Nchan=384
@@ -330,22 +360,9 @@ def chan_map(dp=None, y_orig='surface', probe_version=None):
             cm = np.vstack((cm, cm_el+vert*(i+1)))
         cm=np.hstack([np.arange(Nchan).reshape(Nchan,1), cm])
 
-    else:
-        probe_version='local'
-
-    if probe_version=='local':
-        if dp is None:
-            raise ValueError("dp argument is not provided - when channel map is \
-                             atypical and probe_version is hence called 'local', \
-                             the datapath needs to be provided to load the channel map.")
-        c_ind=np.load(dp/'channel_map.npy');cp=np.load(dp/'channel_positions.npy')
-        cm=npa(np.hstack([c_ind.reshape(max(c_ind.shape),1), cp]), dtype=np.int32)
-
-    if y_orig=='surface':
-        cm[:,1:]=cm[:,1:][::-1]
-
     return cm
 
+
 #%% Binary file I/O, including sync channel
 
 def get_binary_file_path(dp, filt_suffix='ap', absolute_path=True):
@@ -688,7 +705,8 @@ def preprocess_binary_file(dp=None, filt_key='ap', fname=None, target_dp=None, m
                        ADC_realign = False, median_subtract=True, f_low=None, f_high=300, order=3,
                        filter_forward=True, filter_backward=False,
                        spatial_filt=False, whiten = False, whiten_range=32,
-                       again_Wrot=False, verbose=False):
+                       again_Wrot=False, verbose=False, again_if_preprocessed_filename=False,
+                       delete_original_data=False, data_deletion_double_check=False):
     """Creates a preprocessed copy of binary file at dp/fname_filtered.bin,
     and moves the original binary file to dp/original_data.fname.bin.
 
@@ -723,6 +741,9 @@ def preprocess_binary_file(dp=None, filt_key='ap', fname=None, target_dp=None, m
     - spatial_filt: bool, whether to high pass filter across channels at 0.1 Hz
     - whiten: bool, whether to whiten across channels
     - verbose: bool, whether to print extra information
+    - again_if_preprocessed_filename: bool, whether to re-filter if the name of the binary file has hallmarks of preprocessing.
+    - delete_original_data: bool, whether to delete the original binary file after filtering
+    - data_deletion_double_check: bool, must ALSO be true to alow deletion of the original binary file
     """
 
     # Parameters check
@@ -741,6 +762,12 @@ def preprocess_binary_file(dp=None, filt_key='ap', fname=None, target_dp=None, m
     # Fetch binary file name, define target file name
     if fname is None:
         fname = get_binary_file_path(dp, filt_key, True)
+
+    if not again_if_preprocessed_filename:
+        if detected_preprocessed_fname(fname):
+            print(f"\nBinary file name {fname} seems to have already been preprocessed. If you wish to re-process it, set 'again_if_preprocessed_filename' to True.\n")
+            return fname
+
     fname=Path(fname)
     dp = fname.parent
     if target_dp is None:
@@ -752,7 +779,7 @@ def preprocess_binary_file(dp=None, filt_key='ap', fname=None, target_dp=None, m
     filtered_fname, message = \
         make_preprocessing_fname(fname, ADC_realign, median_subtract,
                             f_low, f_high, filter_forward, filter_backward,
-                            whiten, whiten_range, spatial_filt)
+                            whiten, whiten_range, spatial_filt)    
     print(message)
 
     # fetch metadata
@@ -892,12 +919,23 @@ def preprocess_binary_file(dp=None, filt_key='ap', fname=None, target_dp=None, m
             if (dp/'whitening_mat.npy').exists():
                 shutil.copy(dp/'whitening_mat.npy', orig_dp/'whitening_mat.npy')
 
+    if delete_original_data:
+        assert data_deletion_double_check,\
+        "WARNING you are attempting to delete the original binary file - if you wish to proceed, you must also set 'data_deletion_double_check' to True."
+        if move_orig_data:
+            shutil.rmtree(dp/'original_data')
+        else:
+            os.remove(fname)
+    else:
+        if data_deletion_double_check:
+            print("WARNING you attempted to delete the original binary file - 'delete_original_data' was not set to True, so the deletion was cancelled.")
+
     return target_dp/filtered_fname
 
 def make_preprocessing_fname(fname, ADC_realign, median_subtract,
                             f_low, f_high, filter_forward, filter_backward,
                             whiten, whiten_range, spatial_filt):
-
+        
     filter_suffix = ""
     message = ""
     if ADC_realign:
@@ -921,6 +959,15 @@ def make_preprocessing_fname(fname, ADC_realign, median_subtract,
 
     return filtered_fname, message
 
+def detected_preprocessed_fname(fname):
+
+    fname_str = str(fname)
+    patterns = ["adcshift", "medsub", "tempfilt", "whit"]
+    for pat in patterns:
+        if pat in fname_str:
+            return True
+    return False
+
 #%% paqIO file loading utilities
 
 def paq_read(file_path):
